use serde_json;
use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize,Debug)]
pub struct Actor {
    #[serde(rename = "type")]
    pub object_type: ObjectType,
    pub name: String, // The username
    #[serde(rename = "preferredUsername")]
    pub preferred_username: String, // The @ handle
    pub following: String, // Link to list of users which are followed by them
    pub followers: String, // Link to list of users which follow them
    pub inbox: String, // Link to collection where notes will be posted to
    pub outbox: String, // Link to notes by this user
    pub summary: Option<String>, // Bio
    pub icon: Option<Icon>,  // Link to pfp
}

#[derive(Serialize, Deserialize,Debug,PartialEq)]
pub struct Icon {
    #[serde(rename = "type")]
    media_type: String,
    #[serde(rename = "mediaType")]
    media_mime_type: String,
    url: String,

}
#[derive(Serialize, Deserialize,Debug,PartialEq)]
pub enum ObjectType {
    Person,
    Like,
    Note,
    Tombstone,
}

pub enum Object {
    Person(Actor),
    Like,
    Note,
    Tombstone,
}

pub fn json_to_actor(input: &String)->Actor {
    let return_data: Actor = serde_json::from_str(&input).unwrap();
    return_data
}
#[cfg(test)]
mod tests {

    use super::*;
    use std::fs::read_to_string;
    #[test]
    fn parse_json_into_actor() {
        let input_string = read_to_string("test-json.json").unwrap();
        let actor: Actor = json_to_actor(&input_string);

        assert_eq!(actor.object_type,ObjectType::Person);
        assert_eq!(actor.name,"Sebastian Jambor".to_string());
        assert_eq!(actor.preferred_username,"crepels");
        assert_eq!(actor.following,"https://mastodon.social/users/crepels/following".to_string());
        assert_eq!(actor.followers,"https://mastodon.social/users/crepels/followers".to_string());
        assert_eq!(actor.inbox,"https://mastodon.social/users/crepels/inbox".to_string());
        assert_eq!(actor.outbox,"https://mastodon.social/users/crepels/outbox".to_string());
        assert_eq!(actor.summary,Some("<p>I created a systemd playground to help people learn systemd.</p>".to_string()));
        assert_eq!(actor.icon,Some(Icon{media_type: "Image".to_string(), media_mime_type: "image/png".to_string(), url: "https://files.mastodon.social/accounts/avatars/108/227/485/389/961/502/original/1213d525278ae01d.png".to_string()}))
    }
}
